// test for randomized.js

Store = function(arr) {
 return {
   getAt : function(n) {
     return arr[n];
   },
   data: arr
 };
};

describe("randomize a collection", function() {
  var store = new Store(["un","deux","trois","quart","cinq","six"]);
  
  rand = Randomized(store, "getAt");
  rand.init(store.data.length);
  
  it("should return a fixed yet random member at() a valid index", function() {
    expect(store.data).toContain(rand.at(2));
    expect(rand.at(1)).toBe(rand.at(1));
  });
  
  it("next() should return the first entry right after init()", function() {
    rand.init(store.data.length);
    expect(rand.next()).toBe(rand.at(0));
  });
  
  it("next() should go back to the first entry after the last", function() {
    rand.init(store.data.length);
    for (var i = 0; i < store.data.length; i++) {
      rand.next();
    }
    expect(rand.next()).toBe(rand.at(0));
  });
  
  it("next() should invoke callback when return last entry", function() {
    var epilogue = jasmine.createSpy();
    rand.init(store.data.length);
    for (var i = 0; i < store.data.length - 1; i++) {
      rand.next(epilogue);
    }
    expect(epilogue).not.toHaveBeenCalled();
    rand.next(epilogue);
    expect(epilogue).toHaveBeenCalled();
  });
 
  it("should return a random index when no index method", function() {
    var rand2 = Randomized(store);
    for (var i = 0; i < store.data.length; i++) {
      expect(typeof rand2.at(i) === 'number').toBeTruthy();
    }
  });

  it("should return a random index without a collection", function() {
    var rand2 = Randomized();
    for (var i = 0; i < store.data.length; i++) {
      expect(typeof rand2.at(i) === 'number').toBeTruthy();
    }
  });

  it("prev() should return the last entry right after init()", function() {
    rand.init(store.data.length);
    expect(rand.prev()).toBe(rand.at(store.data.length - 1));
  });
  
  it("prev() returns next()'s previous result vice versa", function() {
    var first, 
        callback = jasmine.createSpy();
    rand.init(store.data.length);
    first = rand.next();
    expect(first).toBe(rand.prev());
    expect(first).toBe(rand.next());
  });

  it("prev() should invoke callback once returned the 1st entry", function() {
    var callback = jasmine.createSpy();
    rand.init(store.data.length);
    rand.next();
    expect(callback).not.toHaveBeenCalled();
    rand.prev(callback);
    expect(callback).toHaveBeenCalled();
  });

});
